require "proboscis_cli/version"
require 'httparty'
require 'json'
require "base64"

class Machine
  def self.port(environment, type, subpath)
    ENV["proboscis_#{environment}_target_port"] || "22"
  end
end

class Server < Machine
  def self.ip(auth_code, host, environment, subdomain, type, subpath)
    response = HTTParty.get("#{host}/api/api/hives/#{subdomain}/#{type}", headers: {"X-Authorization" => auth_code})
    if response.code != 200
      puts "Unable to fetch details for domain #{subdomain}"
      exit -1
    end
    hive = JSON::parse(response.body)
    @ipAddress = hive["ip_address"]
  end
end

class CashServer < Server
  def self.port(environment, type, subpath)
    subpath == 'araneae' ? "1729" : "22"
  end

  def self.param(environment, lookup)
    ENV["araneae_#{environment}_#{lookup}"]
  end

  def self.ip(auth_code, host, environment, subdomain, type, subpath)
    super
    if subpath == 'araneae'
      return @ipAddress
    end
    res = HTTParty.get("https://#{subdomain}.delium.io/api/admin/hosts", headers: {'X-Auth-Key': self.param(environment, 'token')})
    if res.code != 200
      puts "Failed to get subpath details. #{res.code}"
      exit -1
    end
    subservers = JSON::parse(res.body)
    @ipAddress = subservers.select {|s| s['name'] == subpath}[0]['host']
    @ipAddress
  end
end

class Worker < Machine
  def self.ip(auth_code, host, environment, subdomain, type, subpath)
    @auth_code = auth_code
    @host = host
    self.client_list environment
    client_id = @client_list.select {|c| c["domain"] == subdomain}
    if client_id.empty?
      puts "Unable to find client with subdomain #{subdomain}"
      exit -1
    end
    self.client environment, client_id.first
    self.worker_ip subpath
  end

  def self.worker_ip(subpath)
    if(!@infra_info["WORKER_IP"])
      puts "Unable to find target WORKER_IP in #{@infra_info}. Is the machine up?"
    end
    @infra_info["WORKER_IP"]
  end

  def self.client_list(environment)
    response = HTTParty.get("#{@host}/api/api/clients", headers: {"X-Authorization" => @auth_code})
    if response.code != 200
      puts "Unable to get the client list.."
      exit -1
    end
    @client_list = JSON::parse(response.body)
  end

  def self.client(environment, client_id)
    response = HTTParty.get("#{@host}/api/api/clients/#{client_id["id"]}/products", headers: {"X-Authorization" => @auth_code})
    if response.code != 200
      puts "Unable to fetch details of client with id #{client_id}"
      exit -1
    end
    infra_info = JSON::parse(response.body)
    if infra_info.empty?
      puts "No Products Mapped!"
      exit -1
    end
    @infra_info = JSON::parse(infra_info["infrastructureInfo"])
   end
end

class CashWorker < Worker
  def self.worker_ip(subpath)
    key = "#{subpath.gsub('_', '-')}_WORKER_IP"
    puts @infra_info
    if(!@infra_info[key])
      puts "Unable to find target WORKER_IP in #{@infra_info}. Is the machine up?"
    end
    @infra_info[key]
  end
end

class Proboscis
  def self.host(environment)
    ENV["proboscis_#{environment}"]
  end

  def self.param(environment, lookup)
    ENV["proboscis_#{environment}_#{lookup}"]
  end

  def self.connect(environment, subdomain, target, type, subpath='araneae')
    type = type == 'b' ? 'BUSY_BEE' : (type == 'i' ? 'INSTANT_HONEY' : 'CASH_REPLENISHMENT')
    @host = self.host(environment)
    self.login environment
    machine = self.get_machine target, type
    ip = machine.ip @auth_code, @host, environment, subdomain, type, subpath
    if(!ip)
      puts "Unable to find target #{ip}. Is the machine up?"
      exit -1
    end
    user = ENV["proboscis_#{environment}_target_user"] || "root"
    port = machine.port environment, type, subpath
    puts "Executing ssh #{user}@#{ip} -p #{port}"
    system "ssh #{user}@#{ip} -p #{port}"
  end

  def self.get_machine(target, type)
    puts type
    if type == 'CASH_REPLENISHMENT'
      target == 'server' ? CashServer : CashWorker
    else
      target == 'server' ? Server : Worker
    end
  end

  def self.login(environment)
    response = HTTParty.post("#{@host}/api/login", body: {phoneNumber: self.param(environment, 'user'), password: self.param(environment, 'pass')}.to_json, headers: { 'Content-Type' => 'application/json' })
    if response.code != 200
      puts "Unable to login to proboscis.."
      exit -1
    end
    @auth_code = Base64.strict_encode64(JSON::parse(response.body)["accessToken"])
  end
end
